<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Validation\ValidationException;
use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Facades\Hash;

class AuthController extends Controller
{
   public function login(Request $request)
{
    $request->validate([
        'email' => 'required|email',
        'password' => 'required|string',
    ]);

    // Fetch user based on email
    $user = User::where('email', $request->email)
        ->with('role') // Eager load the role relationship
        ->first();

    // Check if user exists
    if (!$user) {
        return response()->json([ 'status' => 401, 'message' => 'User Not Found!']);
    }

    // Check if password matches
    if (!Hash::check($request->password, $user->password)) {
        return response()->json([ 'status' => 401, 'message' => 'Password does not match! Please check your login credential!']);
    }

    // Check user status
    if ($user->status != 1) {
        return response()->json([ 'status' => 401, 'message' => 'User account is inactive!']);
    }

    // Determine role name
    $roleNames = [
        1 => 'Admin',
        2 => 'Super',
        3 => 'Senior',
        4 => 'Client',
        5 => 'Worker',
    ];

    $rolename = $roleNames[$user->role_id] ?? 'Unknown';

    $latestProjectId = 0;

    // If the user is a worker, fetch the latest project where the user is in assign_workers
    if ($rolename === 'Worker') {
        $latestProject = \App\Models\Project::whereJsonContains('assign_workers', (string) $user->id)
            ->orderBy('id', 'desc')
            ->first();

        $latestProjectId = $latestProject ? $latestProject->id : null;
    }else{
        $latestProjectId = $latestProjectId;
    }

    // Respond with user data
    return response()->json([
        'status' => 200,
        'message' => 'User Login successfully',
        'data' => [
            'id' => $user->id,
            'username' => $user->name,
            'email' => $user->email,
            'phone_number' => $user->phone_number,
            'status' => $user->status == 1 ? 'Active' : 'Inactive',
        ],
    ], 200);
}


public function signup(Request $request)
{
    try {
        // Validate the request
        $request->validate([
            'name' => 'required|string',
            'email' => 'required|email|unique:users,email',
            'password' => 'required|string',
            'phone_number' => 'required|string',
        ]);

        // Create the user
        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password), // Hash the password
            'phone_number' => $request->phone_number,
            'status'=>1
        ]);

        // Return a success response
        return response()->json([
            'status' => 200,
            'message' => 'User registered successfully.',
            'data' => [
                'id' => $user->id,
                'name' => $user->name,
                'email' => $user->email,
                'phone_number' => $user->phone_number,
                'status' => $user->status == 1 ? 'Active' : 'Inactive',
            ],
        ], 200);

    } catch (ValidationException $e) {
        // Return validation errors as JSON
        return response()->json([
            'status' => 403,
            'message' => 'Validation errors occurred.',
            'errors' => $e->errors(), // Include all validation errors
        ], 403);
    }
}

}
