<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Project;

use App\Models\User;
use App\Models\Role;
use App\Models\Task;
use Carbon\Carbon; // For working with dates

class ProjectController extends Controller
{
   public function list(Request $request)
{
    $request->validate([
        'user_id' => 'required|integer',
    ]);

    $userId = (string) $request->user_id;

    // Fetch the user to get the role_id
    $user = User::find($userId);

    if (!$user) {
        return response()->json([
            'status' => 404,
            'message' => 'User not found',
        ]);
    }

    // Query projects where the user_id matches in any of the relevant fields
    $projects = Project::whereJsonContains('assign_supervisor', $userId)
        ->orWhereJsonContains('assign_senior_supervisor', $userId)
        ->orWhereJsonContains('assign_clients', $userId)
        ->orWhereJsonContains('assign_workers', $userId)
        ->get()
        ->map(function ($project) {
            return [
                'id' => $project->id,
                'name' => $project->name, // Assuming the project has a 'name' attribute
                'address' => $project->address != "" ? $project->address : "",
                'assign_supervisor' => $this->getUserDetails($project->assign_supervisor),
                'assign_senior_supervisor' => $this->getUserDetails($project->assign_senior_supervisor),
                'assign_clients' => $this->getUserDetails($project->assign_clients),
                'assign_workers' => $this->getUserDetails($project->assign_workers),
            ];
        });

    // Return the response
    return response()->json([
        'status' => 200,
        'message' => 'Projects fetched successfully',
        'projects' => $projects,
    ], 200);
}


public function getProjectsTasklist(Request $request)
{
    // $request->validate([
    //     'user_id' => 'required|integer',
    //     'project_id' => 'required|integer',
    //     'date' => 'required|date_format:d-m-Y', // Validate the date format
    // ]);

    $userId = $request->user_id;
    $projectId = $request->project_id;
    $date = $request->date;

    // Convert the date to Y-m-d format for database compatibility
    $formattedDate = Carbon::createFromFormat('d-m-Y', $date)->format('Y-m-d');

    // Check if the project exists
    $project = Project::find($projectId);

    if (!$project) {
        return response()->json([
            'status' => 404,
            'message' => 'Project not found',
        ]);
    }

    // Verify the user is associated with the project
    $userInProject = Project::where('id', $projectId)
        ->where(function ($query) use ($userId) {
            $query->whereJsonContains('assign_supervisor', $userId)
                ->orWhereJsonContains('assign_senior_supervisor', $userId)
                ->orWhereJsonContains('assign_clients', $userId)
                ->orWhereJsonContains('assign_workers', $userId);
        })
        ->exists();

    // if (!$userInProject) {
    //     return response()->json([
    //         'status' => 403,
    //         'message' => 'User is not associated with this project',
    //     ], 403);
    // }

    // Transform the project to include only necessary user-related fields
    $projectDetails = [
        'id' => $project->id,
        'assign_supervisor' => $this->getUserDetails($project->assign_supervisor),
        'assign_senior_supervisor' => $this->getUserDetails($project->assign_senior_supervisor),
        'assign_clients' => $this->getUserDetails($project->assign_clients),
        'assign_workers' => $this->getUserDetails($project->assign_workers),
    ];

    // Fetch tasks
    $tasks = Task::where('project_id', $projectId)
        ->whereDate('task_date', $formattedDate)
        ->get()
        ->map(function ($task) {
            $taskData = $task->toArray();

            // Add user role, role name, and user_name based on task's user_id
            $user = User::find($task->user_id);
            if ($user) {
                $role = Role::find($user->role_id);
                $taskData['designation'] = $role ? $role->name : null;
                $taskData['user_name'] = $user->name . ' ' . $user->last_name;
            } else {
                $taskData['designation'] = null;
                $taskData['user_name'] = null;
            }

             // Decode images and generate full paths with a specific base URL
        $imagePaths = json_decode($task->image, true) ?? [];
        $baseImagePath = 'https://jenson.co.in/awilli/storage/app/public/';
        $taskData['image'] = array_map(function ($path) use ($baseImagePath) {
            return $baseImagePath . $path; // Use the specified base URL
        }, $imagePaths);


            // Fetch the task's associated project
            $project = Project::find($task->project_id);

            // Initialize supervisors and senior supervisors
            $taskData['assign_supervisors'] = [];
            $taskData['assign_senior_supervisors'] = [];

            if ($project) {
                // Decode and fetch supervisor and senior supervisor details
                $assignSupervisors = is_array($project->assign_supervisor) ? $project->assign_supervisor : json_decode($project->assign_supervisor, true);
                $assignSeniorSupervisors = is_array($project->assign_senior_supervisor) ? $project->assign_senior_supervisor : json_decode($project->assign_senior_supervisor, true);

$taskData['assign_supervisors'] = User::whereIn('id', $assignSupervisors)
    ->get()
    ->map(function ($user) {
        return $user->name . ' ' . $user->last_name; // Directly return the concatenated string
    })
    ->toArray();

$taskData['assign_senior_supervisors'] = User::whereIn('id', $assignSeniorSupervisors)
    ->get()
    ->map(function ($user) {
        return $user->name . ' ' . $user->last_name; // Directly return the concatenated string
    })
    ->toArray();
            }

            return $taskData;
        });

    // If no tasks were found, set 'tasks' to an empty object
    if ($tasks->isEmpty()) {
        $tasks = new \stdClass(); // Empty object
    }

    // Return the tasks and filtered project details
    return response()->json([
        'status' => 200,
        'message' => 'Tasks fetched successfully',
        'project' => $projectDetails,
        'tasks' => $tasks,
    ], 200);
}














/*
      
 * Helper function to fetch user details by IDs.
 */
private function getUserDetails($userIds)
{
    if (!$userIds || !is_array($userIds)) {
        return [];
    }

    // Fetch user details by IDs and return them as an array
    return User::whereIn('id', $userIds)
        ->get(['id', 'name', 'last_name'])
        ->map(function ($user) {
            return [
                'id' => $user->id,
                'name' => $user->name . " " . $user->last_name,
            ];
        })
        ->toArray();
}

}
