<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Task;
use App\Models\User;
use App\Models\Role;
use Illuminate\Support\Facades\Storage;
use Carbon\Carbon; // For working with dates


class TaskController extends Controller
{
public function store(Request $request)
{
    // Validate incoming request
    $validated = $request->validate([
        'user_id' => 'required|integer',
        'project_id' => 'required|integer',
        'task_name' => 'required|string|max:255',
        'task_description' => 'required|string',
        'task_date' => 'required|date',
        'task_image.*' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        'task_status' => 'nullable|string', // task_status is now nullable
    ]);

    // If task_status is not provided, set it to "open"
    $taskStatus = $validated['task_status'] ?? 'open';  // Default value is "open"

    // Initialize an array for storing uploaded images paths
    $imagePaths = [];

    if ($request->hasFile('task_image')) {
        foreach ($request->file('task_image') as $image) {
            // Store the image in the 'public' disk under 'task_images' folder
            $path = $image->store('task_images', 'public');
            $imagePaths[] = $path;
        }
    }

    // Create the new task
    $task = Task::create([
        'user_id' => $validated['user_id'],
        'project_id' => $validated['project_id'],
        'task_name' => $validated['task_name'],
        'task_description' => $validated['task_description'],
        'task_date' => date('Y-m-d', strtotime($validated['task_date'])),
        'task_status' => $taskStatus, // Use the default or provided task_status
        'image' => json_encode($imagePaths), // Store image paths as JSON
    ]);

    // Prepare the response with formatted task_date
    $taskData = $task->toArray();
    $taskData['task_date'] = Carbon::parse($taskData['task_date'])->format('d-m-Y');

    // Map the image paths to full URLs
    $imagePaths = json_decode($taskData['image'], true);
    if ($imagePaths) {
        // Use asset() to generate full URLs for images
        $taskData['image'] = array_map(function ($path) {
            return 'https://elecorepair.site/storage/app/public/' . $path; // Use the 'public/storage' symlink
        }, $imagePaths);
    }

    // Return success response
    return response()->json([
        'status' => 200,
        'message' => 'Task created successfully.',
        'data' => $taskData,
    ], 200);
}


public function show(Request $request)
{
    $request->validate([
        'id' => 'required|integer',
    ]);

    $id = (string) $request->id;

    // Find the task by ID
    $task = Task::find($id);

    // If task is not found, return 404 response
    if (!$task) {
        return response()->json([
            'success' => false,
            'message' => 'Task not found',
        ]);
    }

    // Process the image paths to include the full URL
   $imagePaths = json_decode($task['image'], true);
    if ($imagePaths) {
        $task['image'] = array_map(function ($path) {
            return 'https://elecorepair.site/storage/app/public/' . $path; // Relative URL
        }, $imagePaths);
    }

    // Prepare the task data with updated image URLs
    $taskData = $task->toArray();
    $taskData['image'] = $imagePaths;

    // Return the task data in the response
    return response()->json([
        'status' => 200,
        'data' => $taskData,
    ], 200);
}



public function update(Request $request)
{
    // Validate incoming request
    $validated = $request->validate([
        'id' => 'required|integer|exists:tasks,id',
        'user_id' => 'nullable|integer',
        'project_id' => 'nullable|integer',
        'task_name' => 'nullable|string|max:255',
        'task_description' => 'nullable|string',
        'task_date' => 'nullable|date',
        'task_image.*' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        'task_status' => 'nullable|string',
    ]);

    // Find the task by ID
    $task = Task::find($validated['id']);
    if (!$task) {
        return response()->json([
            'success' => false,
            'message' => 'Task not found',
        ], 404);
    }

    $fieldsUpdated = false; // Flag to check if any field is updated

    // Compare and update fields if they differ
    if (isset($validated['user_id']) && $task->user_id !== $validated['user_id']) {
        $task->user_id = $validated['user_id'];
        $fieldsUpdated = true;
    }
    if (isset($validated['project_id']) && $task->project_id !== $validated['project_id']) {
        $task->project_id = $validated['project_id'];
        $fieldsUpdated = true;
    }
    if (isset($validated['task_name']) && $task->task_name !== $validated['task_name']) {
        $task->task_name = $validated['task_name'];
        $fieldsUpdated = true;
    }
    if (isset($validated['task_description']) && $task->task_description !== $validated['task_description']) {
        $task->task_description = $validated['task_description'];
        $fieldsUpdated = true;
    }
    if (isset($validated['task_date'])) {
        $newDate = date('Y-m-d', strtotime($validated['task_date']));
        if ($task->task_date !== $newDate) {
            $task->task_date = $newDate;
            $fieldsUpdated = true;
        }
    }
    if (isset($validated['task_status']) && $task->task_status !== $validated['task_status']) {
        $task->task_status = $validated['task_status'];
        $fieldsUpdated = true;
    }

    // Handle image upload
    if ($request->hasFile('task_image')) {
        $oldImages = json_decode($task->image, true) ?? [];
        foreach ($oldImages as $image) {
            Storage::disk('public')->delete($image); // Delete old images
        }

        $imagePaths = [];
        foreach ($request->file('task_image') as $image) {
            $path = $image->store('task_images', 'public');
            $imagePaths[] = $path;
        }

        $task->image = json_encode($imagePaths);
        $fieldsUpdated = true;
    }

    // Save updated task only if any field was changed
    if ($fieldsUpdated) {
        $task->save();
    }

    // Prepare the response
    $taskData = $task->toArray();
    $taskData['task_date'] = Carbon::parse($taskData['task_date'])->format('d-m-Y');

    // Map image paths to relative URLs
    $imagePaths = json_decode($taskData['image'], true);
    if ($imagePaths) {
        $taskData['image'] = array_map(function ($path) {
            return 'https://elecorepair.site/storage/app/public/' . $path;
        }, $imagePaths);
    }

    // Return response
    return response()->json([
        'status' => 200,
        'message' => $fieldsUpdated ? 'Task updated successfully.' : 'No changes detected.',
        'data' => $taskData,
    ], 200);
}



// public function getTasksByCurrentMonth(Request $request)
// {
//     // Get the month and year from the request, or use Carbon to get the current month and year
//     $currentMonth = $request->month ?? Carbon::now()->month; // Use request month or current month
//     $currentYear = $request->year ?? Carbon::now()->year;   // Use request year or current year
//     $user_id = $request->user_id;
//     $project_id = $request->project_id;

//     // Fetch tasks where task_date matches the provided or current month and year, ordered by task_date in ascending order
//     $tasks = Task::whereYear('task_date', $currentYear)
//                  ->whereMonth('task_date', $currentMonth)
                
//                  ->where('project_id', $project_id)
//                   ->where('user_id', $user_id)
//                  ->orderBy('task_date', 'asc')  // Ordering by task_date ascending
//                  ->get();

//     // If no tasks are found, return a 404 response
//     if ($tasks->isEmpty()) {
//         return response()->json([
//             'status' => 404,
//             'message' => 'No tasks found for the selected month and year.',
//             'data' => [],
//         ], 404);
//     }

//     // Map the tasks to the desired structure
//     $tasksData = $tasks->map(function ($task) {
//         // Format the task date as 'd-m-Y'
//         $taskDate = Carbon::parse($task->task_date)->format('d-m-Y');
        
//         // Prepare task data with the required fields
//         return [
//             'id' => $task->id,
//             'user_id' => $task->user_id,
//             'project_id' => $task->project_id,
//             'task_date' => $taskDate, // Add formatted task date
//         ];
//     });

//     // Return tasks data in the response
//     return response()->json([
//         'status' => 200,
//         'message' => 'Tasks fetched successfully.',
//         'data' => $tasksData,
//     ], 200);
// }


// public function getTasksByCurrentMonth(Request $request)
// {
//     // Get the month and year from the request, or use Carbon to get the current month and year
//     $currentMonth = $request->month ?? Carbon::now()->month; // Use request month or current month
//     $currentYear = $request->year ?? Carbon::now()->year;   // Use request year or current year
//     $user_id = $request->user_id;
//     $project_id = $request->project_id;

//     // Fetch the user's role based on user_id
//     $user = User::find($user_id);
//     if (!$user) {
//         return response()->json([
//             'status' => 404,
//             'message' => 'User not found.',
//             'data' => [],
//         ], 404);
//     }

//     $roleId = $user->role_id;

//     // Build the query
//     $query = Task::whereYear('task_date', $currentYear)
//                  ->whereMonth('task_date', $currentMonth)
//                  ->where('project_id', $project_id);

//     // Apply the user_id condition only if role_id is not 4
//     if ($roleId !== 4) {
//         $query->where('user_id', $user_id);
//     }

//     // Fetch the tasks ordered by task_date in ascending order
//     $tasks = $query->orderBy('task_date', 'asc')->get();

//     // If no tasks are found, return a 404 response
//     if ($tasks->isEmpty()) {
//         return response()->json([
//             'status' => 404,
//             'message' => 'No tasks found for the selected month and year.',
//             'data' => [],
//         ], 404);
//     }

//     // Map the tasks to the desired structure
//     $tasksData = $tasks->map(function ($task) {
//         // Format the task date as 'd-m-Y'
//         $taskDate = Carbon::parse($task->task_date)->format('d-m-Y');
        
//         // Prepare task data with the required fields
//         return [
//             'id' => $task->id,
//             'user_id' => $task->user_id,
//             'project_id' => $task->project_id,
//             'task_date' => $taskDate, // Add formatted task date
//         ];
//     });

//     // Return tasks data in the response
//     return response()->json([
//         'status' => 200,
//         'message' => 'Tasks fetched successfully.',
//         'data' => $tasksData,
//     ], 200);
// }


public function getTasksByCurrentMonth(Request $request)
{
    // Get the month and year from the request, or use Carbon to get the current month and year
    $currentMonth = $request->month ?? Carbon::now()->month;
    $currentYear = $request->year ?? Carbon::now()->year;
    $user_id = $request->user_id;
    $project_id = $request->project_id;

    // Fetch the user's role based on user_id
    $user = User::find($user_id);
    if (!$user) {
        return response()->json([
            'status' => 404,
            'message' => 'User not found.',
            'data' => [],
        ], 404);
    }

    $roleId = $user->role_id;

    // Build the query
    $query = Task::whereYear('task_date', $currentYear)
                 ->whereMonth('task_date', $currentMonth)
                 ->where('project_id', $project_id);

    // Apply the user_id condition only if role_id is not 4
    if ($roleId !== 4) {
        $query->where('user_id', $user_id);
    }

    // Fetch tasks ordered by task_date
    $tasks = $query->orderBy('task_date', 'asc')->get();

    // If no tasks are found, return a 404 response
    if ($tasks->isEmpty()) {
        return response()->json([
            'status' => 404,
            'message' => 'No tasks found for the selected month and year.',
            'data' => [],
        ], 404);
    }

    // Keep only the first task for each unique task_date
    $uniqueTasks = $tasks->unique('task_date');

    // Format the unique tasks into the desired response structure
    $tasksData = $uniqueTasks->map(function ($task) {
        return [
            'id' => $task->id,
            'user_id' => $task->user_id,
            'project_id' => $task->project_id,
            'task_date' => Carbon::parse($task->task_date)->format('d-m-Y'),
        ];
    });

    // Return tasks data in the response
    return response()->json([
        'status' => 200,
        'message' => 'Tasks fetched successfully.',
        'data' => $tasksData->values(), // Ensure numeric indexing
    ], 200);
}


public function deletetaskbyid(Request $request){
    // Validate the incoming request
    $validated = $request->validate([
        'id' => 'required|integer|exists:tasks,id', // Ensure the task ID exists
    ]);

    // Find the task by ID
    $task = Task::find($validated['id']);

    if (!$task) {
        return response()->json([
            'status' => 404,
            'message' => 'Task not found.',
        ], 404);
    }

   
    // Delete associated images
    $imagePaths = json_decode($task->image, true);
    if ($imagePaths) {
        foreach ($imagePaths as $imageUrl) {
            // Remove the base URL to get the relative path
            $relativePath = str_replace('https://elecorepair.site/storage/app/public/', '', $imageUrl);

            // Delete the file from the storage
            Storage::disk('public')->delete($relativePath);
        }
    }

    // Delete the task
    $task->delete();

    // Return a success response
    return response()->json([
        'status' => 200,
        'message' => 'Task deleted successfully.',
    ], 200);
}




}
