<?php
namespace App\Http\Controllers;

use App\Models\Project;
use App\Models\User;
use App\Models\Role;
use Illuminate\Http\Request;

class ProjectController extends Controller
{
public function index()
{
    $projects = Project::all();

    $projects->map(function ($project) {
        // Ensure $project->assign_supervisor is always an array
        $supervisorIds = is_array($project->assign_supervisor) ? $project->assign_supervisor : json_decode($project->assign_supervisor, true);
        $seniorSupervisorIds = is_array($project->assign_senior_supervisor) ? $project->assign_senior_supervisor : json_decode($project->assign_senior_supervisor, true);
        $clientIds = is_array($project->assign_clients) ? $project->assign_clients : json_decode($project->assign_clients, true);
        $workerIds = is_array($project->assign_workers) ? $project->assign_workers : json_decode($project->assign_workers, true);

        // Fetch related users and ensure they are collections
        $project->assign_supervisors = collect(User::whereIn('id', $supervisorIds)->get());
        $project->assign_senior_supervisors = collect(User::whereIn('id', $seniorSupervisorIds)->get());
        $project->assign_clients = collect(User::whereIn('id', $clientIds)->get());
        $project->assign_workers = collect(User::whereIn('id', $workerIds)->get());

        return $project;
    });

    return view('projects.index', compact('projects'));
}








/**
 * Decodes the JSON field and fetches related users.
 *
 * @param mixed $jsonField
 * @return \Illuminate\Database\Eloquent\Collection
 */
private function decodeAndFetchUsers($jsonField)
{
    // Check if the field is already an array
    if (is_array($jsonField)) {
        $userIds = $jsonField;
    } else {
        // Decode the JSON if it's a string
        $userIds = json_decode($jsonField ?? '[]', true);
    }

    // If json_decode was successful and $userIds is a valid array, fetch the users
    if (json_last_error() === JSON_ERROR_NONE && is_array($userIds)) {
        return User::whereIn('id', $userIds)->get();
    }

    // Return an empty collection if decoding failed or $userIds is not a valid array
    return collect();
}





    public function create()
    {
        // Fetch users based on their roles
        $assignSupervisors = User::where('role_id', 2)->get();
        $assignSeniorSupervisors = User::where('role_id', 3)->get();
        $assignClients = User::where('role_id', 4)->get();
        $assignWorkers = User::whereIn('role_id', [5, 6])->get();
    
        return view('projects.create', compact('assignSupervisors', 'assignSeniorSupervisors', 'assignClients', 'assignWorkers'));
    }
public function store(Request $request)
{
    // Validate the form data
    $validatedData = $request->validate([
        'name' => 'required|string|max:255',
        'code' => 'required|string|max:255',
        'description' => 'required|string',
        'address' => 'required|string',
        'status' => 'required|string',
        'assign_supervisor' => 'nullable|array',
        'assign_senior_supervisor' => 'nullable|array',
        'assign_clients' => 'nullable|array',
        'assign_worker' => 'nullable|array',
    ]);

    // Store the project data
    $project = new Project();
    $project->name = $validatedData['name'];
    $project->code = $validatedData['code'];
    $project->description = $validatedData['description'];
    $project->address = $validatedData['address'];
    $project->status = $validatedData['status'];

    // Directly store the array values, let Laravel handle the JSON encoding for you
    $project->assign_supervisor = $validatedData['assign_supervisor'] ?? [];
    $project->assign_senior_supervisor = $validatedData['assign_senior_supervisor'] ?? [];
    $project->assign_clients = $validatedData['assign_clients'] ?? [];
    $project->assign_workers = $validatedData['assign_worker'] ?? [];

    $project->save();

    // Redirect with success message
    return redirect()->route('projects.index')->with('success', 'Project created successfully!');
}



// ProjectController.php

public function edit($id)
{
    // Fetch the project by ID
    $project = Project::findOrFail($id);

    // Fetch users based on their roles
    $assignSupervisors = User::where('role_id', 2)->get(); // Supervisors
    $assignSeniorSupervisors = User::where('role_id', 3)->get(); // Senior Supervisors
    $assignClients = User::where('role_id', 4)->get(); // Clients
    $assignWorkers = User::whereIn('role_id', [5, 6])->get(); // Workers

    return view('projects.edit', compact('project', 'assignSupervisors', 'assignSeniorSupervisors', 'assignClients', 'assignWorkers'));
}


public function update(Request $request, $id)
{
    // Validate the incoming request data
    $validated = $request->validate([
        'name' => 'required|string|max:255',
        'code' => 'required|string|max:255',
        'description' => 'required|string',
        'address' => 'required|string',
        'status' => 'required|string',
        'assign_supervisor' => 'nullable|array',
        'assign_supervisor.*' => 'exists:users,id', // Supervisors IDs must exist in the 'users' table
        'assign_senior_supervisor' => 'nullable|array',
        'assign_senior_supervisor.*' => 'exists:users,id', // Senior Supervisors IDs must exist
        'assign_clients' => 'nullable|array',
        'assign_clients.*' => 'exists:users,id',  // The client must exist in the 'users' table
        'assign_worker' => 'nullable|array',
        'assign_worker.*' => 'exists:users,id', // Workers IDs must exist in the 'users' table
    ]);

    // Find the project by ID
    $project = Project::findOrFail($id);

    // Update project fields
    $project->name = $validated['name'];
    $project->code = $validated['code'];
    $project->description = $validated['description'];
    $project->address = $validated['address'];
    $project->status = $validated['status'];
    
    // Update the relationships (supervisors, senior supervisors, workers, and clients) as raw arrays
    $project->assign_supervisor = $validated['assign_supervisor'] ?? [];
    $project->assign_senior_supervisor = $validated['assign_senior_supervisor'] ?? [];
    $project->assign_clients = $validated['assign_clients'] ?? [];
    $project->assign_workers = $validated['assign_worker'] ?? [];

    // Save the project updates
    $project->save();

    // Redirect back to the projects index with a success message
    return redirect()->route('projects.index')->with('success', 'Project updated successfully.');
}




    public function destroy(Project $project)
    {
        $project->delete();
        return redirect()->route('projects.index');
    }
}
